@@ -10,6 +10,8 @@
 #   unit-tests
 #   more docs and examples
 #   make normal logs
+#   decimal
+#   3.9
 
 from copy import copy
 from datetime import datetime
@@ -58,7 +60,7 @@ class Trader(object):
     _credit_leverages: List[float]
     deposit_history: List[float]
     year_profit: float
-    _linear: np.ndarray
+    average_growth: np.ndarray
     info: str
     _backtest_out_no_drop: pd.DataFrame
     backtest_out: pd.DataFrame
@@ -581,7 +583,6 @@ class Trader(object):
                  commission: float = 0.0,
                  plot: bool = True,
                  print_out: bool = True,
-                 column: str = 'Close',
                  show: bool = True) -> pd.DataFrame:
         """
         testing the strategy.
@@ -590,7 +591,6 @@ class Trader(object):
         :param commission: percentage commission (0 -- 100).
         :param plot: plotting.
         :param print_out: printing.
-        :param column: column of dataframe to backtest
         :param show: show the graph
         returns: pd.DataFrame with data of test
         """
@@ -599,7 +599,7 @@ class Trader(object):
         no_order: bool
         stop_loss: float
         take_profit: float
-        seted: List[Any]
+        converted_element: utils.CONVERTED_TYPE
         diff: float
         lin_calc_df: pd.DataFrame
         high: float
@@ -607,18 +607,17 @@ class Trader(object):
         credit_lev: float
 
         start_bet: float = bet
-        data_column: pd.Series = self.df[column]
+        data_column: pd.Series = self.df['Close']
         data_high: pd.Series = self.df['High']
         data_low: pd.Series = self.df['Low']
         self.deposit_history = [deposit]
-        seted_ = utils.convert(self.returns)
+        converted: utils.CONVERTED_TYPE_LIST = utils.convert(self.returns)
         self.trades = 0
         self.profits = 0
         self.losses = 0
         moneys_open_bet: float = deposit
         money_start: float = deposit
         oldsig = utils.EXIT
-        start_commission: float = commission
 
         e: int
         sig: utils.PREDICT_TYPE
@@ -626,7 +625,7 @@ class Trader(object):
         for e, (sig,
                 stop_loss,
                 take_profit,
-                seted,
+                converted_element,
                 credit_lev,
                 high,
                 low,
@@ -634,14 +633,14 @@ class Trader(object):
                 next_l) in enumerate(zip(self.returns[:-1],
                                          self._stop_losses[:-1],
                                          self._take_profits[:-1],
-                                         seted_[:-1],
+                                         converted[:-1],
                                          self._credit_leverages[:-1],
                                          data_high[:-1],
                                          data_low[:-1],
                                          data_high[1:],
                                          data_low[1:])):
 
-            if seted is not np.nan:
+            if converted_element is not np.nan:
                 if oldsig != utils.EXIT:
                     commission_reuse = 2
                 else:
@@ -651,7 +650,7 @@ class Trader(object):
                     bet = deposit
                 open_price = data_column[e]
                 for i in range(commission_reuse):
-                    deposit -= bet * (start_commission / 100) * credit_lev
+                    deposit -= bet * (commission / 100) * credit_lev
                     if bet > deposit:
                         bet = deposit
                 moneys_open_bet = deposit
@@ -689,7 +688,7 @@ class Trader(object):
             no_order = exit_take_stop
             self.deposit_history.append(deposit)
             oldsig = sig
-            if seted is not np.nan:
+            if converted_element is not np.nan:
                 if sig != utils.EXIT:
                     self.trades += 1
                 if oldsig != utils.EXIT:
@@ -699,11 +698,11 @@ class Trader(object):
                         self.losses += 1
             ignore_breakout = False
 
-        self._linear = utils.get_linear(self.deposit_history)
-        lin_calc_df = pd.DataFrame(self._linear)
-        mean_diff = float(lin_calc_df.diff().mean())
-        self.year_profit = mean_diff * self._profit_calculate_coef
-        self.year_profit = (self.year_profit / money_start) * 100
+        self.average_growth = utils.get_exponential_growth(self.deposit_history)
+        self.year_profit = utils.profit_factor(self.deposit_history) ** (self._profit_calculate_coef - 1)
+        #  Compound interest. View https://www.investopedia.com/terms/c/compoundinterest.asp
+        self.year_profit -= 1  # The initial deposit does not count as profit
+        self.year_profit *= 100  # Percentage
         if self.trades != 0:
             self.winrate = (self.profits / self.trades) * 100
         else:
@@ -720,16 +719,16 @@ winrate: {self.winrate}%"""
         self.returns_strategy_diff[0] = 0
         self._backtest_out_no_drop = pd.DataFrame(
             (self.deposit_history, self._stop_losses, self._take_profits, self.returns,
-             self._open_lot_prices, data_column, self._linear, self.returns_strategy_diff),
+             self._open_lot_prices, data_column, self.average_growth, self.returns_strategy_diff),
             index=[
-                f'deposit ({column})', 'stop loss', 'take profit',
-                'predictions', 'open trade', column,
-                f"linear deposit data ({column})",
+                f'deposit', 'stop loss', 'take profit',
+                'predictions', 'open trade', 'Close',
+                f"average growth deposit data",
                 "returns"
             ]).T
         self.backtest_out = self._backtest_out_no_drop.dropna()
         if plot:
-            loc: pd.Series = self.df[column]
+            loc: pd.Series = self.df['Close']
             self.fig.add_trace(
                 Line(
                     y=self._backtest_out_no_drop['returns'].values,
@@ -775,15 +774,15 @@ winrate: {self.winrate}%"""
                 Line(
                     y=self.deposit_history,
                     line=dict(color=utils.COLOR_DEPOSIT),
-                    name=f'deposit (start: ${money_start})'), 2, 1)
-            self.fig.add_trace(Line(y=self._linear, name='linear'), 2, 1)
+                    name=f'deposit (start: {money_start})'), 2, 1)
+            self.fig.add_trace(Line(y=self.average_growth, name='average growth'), 2, 1)
             preds: Dict[str, List[Union[int, float]]] = {'sellind': [],
                                                          'exitind': [],
                                                          'buyind': [],
                                                          'bprice': [],
                                                          'sprice': [],
                                                          'eprice': []}
-            for e, i in enumerate(seted_):
+            for e, i in enumerate(converted):
                 if i == utils.SELL:
                     preds['sellind'].append(e)
                     preds['sprice'].append(loc[e])
@@ -829,7 +828,6 @@ winrate: {self.winrate}%"""
                        commission: float = 0.0,
                        plot: bool = True,
                        print_out: bool = True,
-                       column: str = 'Close',
                        show: bool = True,
                        limit: int = 1000) -> pd.DataFrame:
         winrates: List[float] = []
@@ -851,7 +849,6 @@ winrate: {self.winrate}%"""
                                 commission=commission,
                                 plot=False,
                                 print_out=False,
-                                column=column,
                                 show=False)
             winrates.append(new_trader.winrate)
             percentage_profits.append(new_trader.year_profit)
@@ -870,14 +867,14 @@ winrate: {self.winrate}%"""
             depo[enum] = np.array(elem[-min(lens_dep):]) / (elem[-min(lens_dep)] / (deposit / len(tickers)))
         self.deposit_history = list(sum(depo))
 
-        self._linear = utils.get_linear(self.deposit_history)
+        self.average_growth = utils.get_exponential_growth(self.deposit_history)
         self.returns_strategy_diff = list(pd.Series(self.deposit_history).diff().values)
         self.returns_strategy_diff[0] = 0
         self._backtest_out_no_drop = pd.DataFrame(
-            (self.deposit_history, self._linear, self.returns_strategy_diff),
+            (self.deposit_history, self.average_growth, self.returns_strategy_diff),
             index=[
-                f'deposit ({column})',
-                f"linear deposit data ({column})",
+                f'deposit',
+                f"average growth deposit data",
                 "returns"
             ]).T
         self.backtest_out = self._backtest_out_no_drop.dropna()
@@ -895,8 +892,8 @@ winrate: {self.winrate}%"""
                 Line(
                     y=self.deposit_history,
                     line=dict(color=utils.COLOR_DEPOSIT),
-                    name=f'deposit (start: ${deposit})'), 2, 1)
-            self.fig.add_trace(Line(y=self._linear, name='linear'), 2, 1)
+                    name=f'deposit (start: {deposit})'), 2, 1)
+            self.fig.add_trace(Line(y=self.average_growth, name='average growth'), 2, 1)
             self.fig.add_trace(
                 Line(
                     y=self.returns_strategy_diff,
@@ -1167,7 +1164,7 @@ winrate: {self.winrate}%"""
                     if not self.__exit_order__:
                         if (open_time + self._sec_interval) - time() < wait_sl_tp_checking:
                             sleep(wait_sl_tp_checking)
-                        utils.logger.debug(f"sleep {wait_sl_tp_checking} seconds")
+                        utils.logger.info(f"sleep {wait_sl_tp_checking} seconds")
 
                         price = self.client.get_ticker_price(ticker)
                         min_ = min(self.__last_stop_loss, self.__last_take_profit)
@@ -1270,7 +1267,7 @@ winrate: {self.winrate}%"""
         :param your_client: trading client
         """
         self.client = your_client
-        utils.logger.info('trader set client')
+        utils.logger.debug('trader set client')
 
     def convert_signal(self,
                        old: utils.PREDICT_TYPE = utils.SELL,
@@ -1280,7 +1277,7 @@ winrate: {self.winrate}%"""
         for pos, val in enumerate(self.returns):
             if val == old:
                 self.returns[pos] = new
-        utils.logger.info(f'trader signals converted: {old} >> {new}')
+        utils.logger.debug(f'trader signals converted: {old} >> {new}')
         return self.returns
 
     def set_open_stop_and_take(self,
@@ -1322,7 +1319,7 @@ winrate: {self.winrate}%"""
                 self._take_profits.append(take_flag)
             if set_stop:
                 self._stop_losses.append(stop_flag)
-        utils.logger.info(f'trader stop loss: {stop_loss}, trader take profit: {take_profit}')
+        utils.logger.debug(f'trader stop loss: {stop_loss}, trader take profit: {take_profit}')
 
     def set_credit_leverages(self, credit_lev: float = 1.0):
         """
